<?php
/**
 * Backup Tool for DokuWiki
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Terence J. Grant<tjgrant@tatewake.com>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'admin.php');
 
/**
 * All DokuWiki plugins to extend the admin function
 * need to inherit from this class
 */
class admin_plugin_backup extends DokuWiki_Admin_Plugin
{
var $state = 0;
var $backup = '';

	/**
	 * Constructor
	 */
	function admin_plugin_backup()
	{
		$this->setupLocale();
	}

	/**
	 * return some info
	 */
	function getInfo()
	{
		return array(
			'author' => 'Terence J. Grant',
			'email'  => 'tjgrant@tatewake.com',
			'date'   => '2006-10-12',
			'name'   => 'Backup Tool Plugin',
			'desc'   => 'A tool to backup your data and configuration.',
			'url'    => 'http://tatewake.com/wiki/projects:backuptool_for_dokuwiki',
		);
	}

	/**
	 * return sort order for position in admin menu
	 */
	function getMenuSort()
	{
		return 999;
	}
	
	/**
	 *  return a menu prompt for the admin menu
	 *  NOT REQUIRED - its better to place $lang['menu'] string in localised string file
	 *  only use this function when you need to vary the string returned
	 */
	function getMenuText()
	{
		return 'Backup Tool';
	}

	/**
	 * handle user request
	 */
	function handle()
	{
		$this->state = 0;
	
		if (!isset($_REQUEST['cmd'])) return;   // first time - nothing to do

		if (!is_array($_REQUEST['cmd'])) return;

		$this->backup = $_REQUEST['backup'];

		if (is_array($this->backup))
		{
			$this->state = 1;
		}
	}

	/**
	 * output appropriate html
	 */
	function html()
	{
		global $conf;

		if ($this->state == 0)
		{
			print $this->plugin_locale_xhtml('intro');

			ptln("<form action=\"".wl($ID)."\" method=\"post\">");
			ptln('  <input type="hidden" name="do"   value="admin" />');
			ptln('  <input type="hidden" name="page" value="'.$this->getPluginName().'" />');
			ptln('  <input type="hidden" name="cmd[backup]" value="true" />');
			print '<center><table class="inline">';
			print '	<tr><th> '.$this->getLang('bt_item_type').' </th><th> '.$this->getLang('bt_add_to_archive').' </th></tr>';
			print '	<tr><td> '.$this->getLang('bt_pages').' </td><td><input type="checkbox" name="backup[pages]" checked/></td></tr>';
			print '	<tr><td> '.$this->getLang('bt_old_revisions_of_pages').' </td><td><input type="checkbox" name="backup[revisions]" checked/></td></tr>';
			print '	<tr><td> '.$this->getLang('bt_meta_data_subscriptions').'</td><td><input type="checkbox" name="backup[subscriptions]" checked/></td></tr>';
			print '	<tr><td> '.$this->getLang('bt_media_files').' </td><td><input type="checkbox" name="backup[media]" checked/></td></tr>';
			print '	<tr><td> '.$this->getLang('bt_wiki_acl_user_config').' </td><td><input type="checkbox" name="backup[config]" checked/></td></tr>';
			print '	<tr><td> '.$this->getLang('bt_templates').'</td><td><input type="checkbox" name="backup[templates]" /></td></tr>';
			print '	<tr><td> '.$this->getLang('bt_plugins').'</td><td><input type="checkbox" name="backup[plugins]" /></td></tr>';
			print '</table>';
			print '<br />';
			print '<p><input type="submit" value="'.$this->getLang('bt_create_backup').'"></p></center>';
			print '</form>';
		}
		else
		{
			$finaldate = date('Ymd-His');
			$finalfile = 'backup-'.$finaldate;

			print $this->plugin_locale_xhtml('outro');

			//remove previous backups
			shell_exec("rm -rf backup*.tar");
			shell_exec("rm -rf backup*.tar.bz2");

			print "<ul>";
			print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." smileys.conf...</div></li>"; ob_flush(); flush();
			shell_exec("tar -cf ".$finalfile.".tar conf/smileys.conf");

			if ($this->backup['pages'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_pages')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar ".$conf['datadir']);
			}
			if ($this->backup['revisions'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_old_revisions_of_pages')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar ".$conf['olddir']);
			}
			if ($this->backup['subscriptions'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_meta_data_subscriptions')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar ".$conf['metadir']);
			}
			if ($this->backup['media'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_media_files')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar ".$conf['mediadir']);
			}
			if ($this->backup['config'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_wiki_acl_user_config')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar conf/acl.auth.php");
				shell_exec("tar -rf ".$finalfile.".tar conf/users.auth.php");
				shell_exec("tar -rf ".$finalfile.".tar conf/local.php");
				shell_exec("tar -rf ".$finalfile.".tar conf/local.protected.php");
			}
			if ($this->backup['templates'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_templates')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar lib/tpl");
			}
			if ($this->backup['plugins'])
			{
				print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_archiving')." ".$this->getLang('bt_plugins')."...</div></li>"; ob_flush(); flush();
				shell_exec("tar -rf ".$finalfile.".tar lib/plugins");
			}

			print "<li class=\"level1\"><div class=\"li\">".$this->getLang('bt_compressing_archive')."...</div></li>"; ob_flush(); flush();
			print "</ul>";

			shell_exec("bzip2 -z -9 ".$finalfile.".tar");
			shell_exec("mv ".$finalfile.".tar.bz2 ".$conf['mediadir']."/".$finalfile.".tar.bz2");

			print $this->plugin_locale_xhtml('download');
			print $this->plugin_render('{{:'.$finalfile.'.tar.bz2}}');

			ob_flush(); flush();
		}
	}

}
